/*******************************************************************************************************
 *        Hochschule Hamm-Lippstadt                                                                    *
 *******************************************************************************************************
 * Modul           : E19a_initDS3231_RealTimeClock                             
 *                                                               
 * Datum           : 27. März 2021                             
 *                                                               
 * Funktion        : Demo zum Stellen der Echtzeit Uhr (RTC3231)      
 *                                                                                                                           
 * Hardware        : UNO PLUS + Multisensor Shield  
 * Implementation  : Arduino IDE 2.0.0-beta.4                    
 *                                                                                             
 *                                                               
 * Author          : Prof. Schneider                             
 *                                                               
 * Bemerkung       : basierend auf Waveshare AlphaBot DS3231_RealTimeClock.ino            
 *                   www.waveshare.com/wiki/AlphaBot   
 *                   https://www.waveshare.com/wiki/Accessory_Shield          
 *                                                               
 * Letzte Änderung : 27. März 2021                             
 *                                                               
 ********************************************************************************************************/

/* Hier finden Sie die aktuelle Uhrzeit https://www.atomuhr.de/ */

#include <Wire.h>
#include <ds3231.h>

/* Konstanten deklarieren */
#define BUFF_MAX 128 // max. Puffergröße

/* Variablen deklarieren */
uint8_t time[8];
struct ts t;      // Struktur für die RTC Befehle

char buff[BUFF_MAX];   // ASCII Puffer


void setup()
{
    Serial.begin(9600);          // Serielle Übertragung mit 9600 Bit/s starten
    Wire.begin();                // Start der I2C Kommunikation mit der RTC
    DS3231_init(DS3231_INTCN);   // Interrupt enable - Ist das nötig?
    Serial.println("SETUP");
    DS3231_get(&t);              // RTC auslesen
    snprintf(buff, BUFF_MAX, "%d.%02d.%02d %02d:%02d:%02d", t.year, t.mon, t.mday, t.hour, t.min, t.sec); // Ausgabe formatieren
    Serial.println(buff);        // Ausgabe der aktuellen Zeit
    t.year = 2021;
    t.mon  =  3;
    t.mday = 28;
    t.hour = 10;
    t.min  = 11;
    t.sec  =  8; // 8 Sekunden nach dem Upload um 00 (ggf. anpassen)
    Serial.println("Neue Uhrzeit:"); 
    DS3231_set(t);                // aktuelle Zeit neu setzen         
}

void loop()
{
    DS3231_get(&t);              // RTC auslesen
    snprintf(buff, BUFF_MAX, "%d.%02d.%02d %02d:%02d:%02d", t.year, t.mon, t.mday, t.hour, t.min, t.sec); // Ausgabe formatieren
    Serial.println(buff);        // Ausgabe der aktuellen Zeit
    delay(1000);                 // Pause in ms
}
