/*******************************************************************************************************
 *        Hochschule Hamm-Lippstadt                                                                    *
 *******************************************************************************************************
 * Modul           : E19b_leseRTC3231                             
 *                                                               
 * Datum           : 27. März 2021                             
 *                                                               
 * Funktion        : Demo zum Auslesen der Echtzeit Uhr (RTC3231)      
 *                                                                                                                           
 * Hardware        : UNO PLUS + Multisensor Shield  
 * Implementation  : Arduino IDE 2.0.0-beta.4                    
 *                                                                                             
 *                                                               
 * Author          : Prof. Schneider                             
 *                                                               
 * Bemerkung       : basierend auf Waveshare AlphaBot DS3231_RealTimeClock.ino            
 *                   www.waveshare.com/wiki/AlphaBot   
 *                   https://www.waveshare.com/wiki/Accessory_Shield          
 *                                                               
 * Letzte Änderung : 27. März 2021                             
 *                                                               
 ********************************************************************************************************/

/* Hier finden Sie die aktuelle Uhrzeit https://www.atomuhr.de/ */

#include <Wire.h>
#include <ds3231.h>

/* Konstanten deklarieren */
#define BUFF_MAX 128  // max. Puffergröße

/* Variablen deklarieren */
uint8_t time[8];
struct ts t;                  // Struktur für die RTC Befehle

char buff[BUFF_MAX];          // ASCII Puffer
unsigned long SystemZeit_u32; // Systemzeit seit Start des Arduino. Läuft nach 50 Tagen über.

void setup()
{
    Serial.begin(9600);   // Serielle Übertragung mit 9600 Bit/s starten
    Wire.begin();         // Start der I2C Kommunikation mit der RTC
    DS3231_init(DS3231_INTCN);        // Interrupt enable - Ist das nötig?
    Serial.println("SETUP");
    DS3231_get(&t);       // RTC auslesen
    snprintf(buff, BUFF_MAX, "%d.%02d.%02d %02d:%02d:%02d", t.year, t.mon, t.mday, t.hour, t.min, t.sec); // Ausgabe formatieren
    Serial.println(buff); // Ausgabe der aktuellen Zeit

}

void loop()
{
    DS3231_get(&t);       // RTC auslesen
    snprintf(buff, BUFF_MAX, "%d.%02d.%02d %02d:%02d:%02d", t.year, t.mon, t.mday, t.hour, t.min, t.sec); // Ausgabe formatieren
    Serial.println(buff); // Ausgabe der aktuellen Zeit

    SystemZeit_u32 = millis(); // Auslesen der Systemzeit in ms
    snprintf(buff, BUFF_MAX, "Systemzeit: %lu ms", SystemZeit_u32);
    Serial.println(buff);      // Ausgabe der Systemzeit
    delay(1000);               // Pause in ms
}
