%****************************************************************
% Modul	          : zeigeArduinoDaten.m                         *
%                                                               *
% Datum           : 27. März 2021                               *
%                                                               *
% Funktion        : Die vom Alphabot gemessene Temperatur wird  *
%                   via RS232 an den PC übertragen und mit      *
%                   MATLAB visualisiert.                        * 
%                                                               *
% Implementation  : MATLAB R2020b                               *
%                                                               *
% Author          : (c) 2021 Prof. Schneider                    *
%                   Alle Rechte vorbehalten.                    *
%                                                               *
% Bemerkung       : basiert auf dem Artikel                     *
% https://www.mathworks.com/help/matlab/import_export/read-streaming-data-from-arduino.html*
%                                                               *
% Letzte Änderung : 27. März 2021                               *
%                                                               *
%****************************************************************

%****************************************************************
%* Vorbereitung: Auf dem AlphaBot läuft E31_Debug2MATLAB.ino    *
%****************************************************************

%% MALTAB initisalisieren
clear all; close all; clc

%% Bestehende serielle Verbindungen löschen
if ~isempty(instrfind)
    fclose(instrfind);
    delete(instrfind);
    disp('Delete open Connection.')
end

stCOMPort = 'COM5'; % COM-Port des Arduino festlegen
nBaudrate = 9600;   % default: 9600 Bit/s
 
hArduinoObj = serialport(stCOMPort,nBaudrate); % Serielle Verbindung aufbauen 

configureTerminator(hArduinoObj,"CR/LF"); % ASCII String, der das Datenende bezeichnet z.B. Carrier Return/Line Feed

flush(hArduinoObj); % Ausgabeobjekt wird persistent verfügbar

% Ergebnisstruktur wird angelegt, 1001 Daten werden aufgezeichnet
hArduinoObj.UserData = struct("Zeit",[],"Data",[],"Count",1,"nDatenEnde",1001); 

% Wird der oben definierte Terminator in der seriellen Kommunikation
% entdeckt, wird die Funktion leseArduinoDaten aufgerufen.
% Achtung: Dies ist ein zyclischer Callback.
configureCallback(hArduinoObj,"terminator",@leseArduinoDaten); 




